@@ -30,6 +30,8 @@ from eukcc import treelineage
 from eukcc.fileoperations import file
 import hashlib
 from ete3 import NCBITaxa
+from pyfaidx import Fasta
+from collections import defaultdict
 
 
 dep = {
@@ -46,6 +48,7 @@ class eukcc:
         # stores all magic numbers and user settings
         self.config = eukinfo(options)
         self.cfg = self.config.cfg
+        self._clean_fasta = None
 
         # set name of run
         self.cfg["name"] = os.path.splitext(os.path.basename(options.fasta))[0]
@@ -69,6 +72,43 @@ class eukcc:
     def writeProcess(self):
         print(self.config.cfg["GeneMark-ES"])
 
+    def get_silent_contig(self, fasta, hits, placements):
+        """
+        given hits and the fasta file, we can calculate how many
+        DNA bp can not be detected as contaminant, because no markers
+        are on these contigs
+        """
+        logging.debug("Calculating silent contig fraction")
+        if fasta is None:
+            logging.debug("As no DNA fasta was given, we can't compute the contig fraction")
+            for placement in placements:
+                placement["max_silent_contamination"] = "NA"
+            return
+
+        # get dict to link profile to contigs
+        profile_2_contigs = defaultdict(set)
+        for row in base.readTSV(hits):
+            profile_2_contigs[row["profile"]].add(row["chrom"])
+
+        # read in fasta once
+        dna_lens = {}
+        for rec in Fasta(fasta):
+            dna_lens[rec.name] = len(str(rec))
+        total_len = sum([v for k, v in dna_lens.items()])
+
+        # for each placement compute the silent fraction
+        for placement in placements:
+            contigs = set()
+            # make union of contigs
+            for profile in placement["set"]:
+                contigs = contigs | profile_2_contigs[profile]
+            # define missing contigs as contig names we did not locate any marker genes on
+            m_contigs = set(dna_lens.keys()) - contigs
+            missing_len = sum([dna_lens[contig] for contig in m_contigs])
+            placement["max_silent_contamination"] = round(missing_len / total_len * 100, 2)
+            logging.debug("The silent fraction could be up to {}%".format(placement["max_silent_contamination"]))
+        return
+
     def estimate(self, hits, outfile, placements):
         hit = {}
         logging.info("Estimating scores now")
@@ -92,6 +132,7 @@ class eukcc:
         # now we can estimate completeness and contamination for each placement
         for i in range(len(placements)):
             s = self.readSet(placements[i]["node"])
+            placements[i]["set"] = s
             # completeness is the overap of both sets
             cmpl = len(singletons & s) / len(s)
             cont = len(multitons & s) / len(s)
@@ -100,12 +141,23 @@ class eukcc:
             placements[i]["completeness"] = round(cmpl * 100, 2)
             placements[i]["contamination"] = round(cont * 100, 2)
 
-        log("Finished estimating")
+        # compute silent fraction per placement and set
+        self.get_silent_contig(self._clean_fasta, hits, placements)
 
+        log("Finished estimating")
+        self.write_outfile(outfile, placements)
+
+        # done
+        return True
+
+    def write_outfile(self, outfile=None, result=None):
+        if outfile is None:
+            outfile = os.path.join(self.cfg["outdir"], "eukcc.tsv")
         # write to output file
         k = [
             "completeness",
             "contamination",
+            "max_silent_contamination",
             "node",
             "n",
             "ngenomes",
@@ -118,7 +170,10 @@ class eukcc:
         ]
         with open(outfile, "w") as f:
             f.write("{}\n".format("\t".join(k)))
-            for p in placements:
+            if result is None:
+                logging.warning("No estimates were written")
+                exit(11)
+            for p in result:
                 # insert the file name
                 p["file"] = self.cfg["name"]
                 # write to file
@@ -126,9 +181,6 @@ class eukcc:
 
         log("Wrote estimates to: {}".format(outfile))
 
-        # done
-        return True
-
     def readSet(self, node):
         profiles = []
         localpath = os.path.join(self.cfg["db"], "sets", "{}.set".format(node))
@@ -177,6 +229,7 @@ class eukcc:
         # concatenate
         if len(profiles) == 0:
             logging.error("We have no profiles to evaluate")
+            self.write_outfile()
             exit(1)
 
         log("{} hmm profiles need to be used for estimations".format(len(profiles)))
@@ -240,27 +293,34 @@ class eukcc:
         logging.debug("Infering lineages now")
         for p in places:
             # get the GCA names
-            children = p["sisters"]
-            # fetch lineages for all
+            children = p.get("sisters")
             lngs = []
-            for c in children:
-                try:
-                    lngs.append(ncbi.get_lineage(taxids[c]))
-                except ValueError as e:
-                    logging.warning(e)
+            if children is not None:
+                # fetch lineages for all
+                for c in children:
+                    try:
+                        lngs.append(ncbi.get_lineage(taxids[c]))
+                    except ValueError as e:
+                        logging.warning(e)
 
             # find common elements:
-            common = set(lngs[0])
-            for l in lngs[1:]:
-                common = common & set(l)
+            if len(lngs) == 0:
+                logging.warning(
+                    "Could not determine common lineage. Please update your ete3 taxonomic information: http://etetoolkit.org/docs/latest/tutorial/tutorial_ncbitaxonomy.html#upgrading-the-local-database"
+                )
+                lng = [1]
+            else:
+                common = set(lngs[0])
+                for l in lngs[1:]:
+                    common = common & set(l)
 
-            # common lineage
-            lng = []
-            for v in lngs[0]:
-                if v not in common:
-                    break
-                # add common elements
-                lng.append(v)
+                # common lineage
+                lng = []
+                for v in lngs[0]:
+                    if v not in common:
+                        break
+                    # add common elements
+                    lng.append(v)
 
             nodetaxid = lng[-1]
             # now we can make it pretty
@@ -326,6 +386,7 @@ class eukcc:
             # log and document failing
             # then stop pipeline
             logging.error("GeneMark-ES failed on this bin")
+            self.write_outfile()
             exit(1)
         elif self.cfg["clean"]:
             # clean temp dirs
@@ -342,24 +403,35 @@ class eukcc:
         # touch files expected for next step
         if self.cfg["touch"]:
             g.touch([bedf, gmesOut])
+        self._clean_fasta = inputfasta
         return (gmesOut, bedf)
 
     def pygmes(self, fasta, db):
         outdir = os.path.join(self.cfg["outdir"], "workfiles", "pygmes")
         faafile = os.path.join(outdir, "predicted_proteins.faa")
         bedfile = os.path.join(outdir, "predicted_proteins.bed")
+        self._clean_fasta = os.path.join(outdir, "gmesclean_{}".format(os.path.basename(fasta)))
         from pygmes import pygmes
 
-        pygmes(fasta, outdir, db=db, clean=True, ncores=self.cfg["ncores"])
+        # check if we need to launch
+        need_run = False
+        if not file().exists(faafile) or not file().exists(bedfile):
+            need_run = True
+        elif file.isnewer(fasta, faafile):
+            need_run = True
+        if need_run:
+            pygmes(fasta, outdir, db=db, clean=True, ncores=self.cfg["ncores"])
         # check if pg worked
         if os.path.exists(faafile) and os.path.exists(bedfile):
             if os.stat(faafile).st_size == 0 or os.stat(bedfile).st_size == 0:
                 logging.warning("No predicted proteins")
+                self.write_outfile()
                 exit(1)
             else:
                 return (faafile, bedfile)
         else:
             logging.warning("No predicted proteins, pyfaidx failed")
+            self.write_outfile()
             exit(1)
 
     def place(self, fasta, bedfile):
@@ -370,9 +442,11 @@ class eukcc:
         # test if we can open the input files first
         if not base.exists(fasta):
             logging.error("Could not open fasta file")
+            self.write_outfile()
             exit(1)
         if not base.exists(bedfile):
             logging.error("Could not open bed file")
+            self.write_outfile()
             exit(1)
 
         # define output files
@@ -423,6 +497,7 @@ class eukcc:
             )
             if pp.lenscmgs == 0 and not self.cfg["touch"]:
                 logging.error("Could not find any marker genes")
+                self.write_outfile()
                 exit(1)
             else:
                 logging.info("Placing proteins in tree")
@@ -434,6 +509,7 @@ class eukcc:
                 )
                 if pplacer_success is False:
                     logging.warning("Pplacer could not finish. Exiting now")
+                    self.write_outfile()
                     exit(1)
 
         # reduce placements to the placements with at least posterior of p
@@ -448,7 +524,10 @@ class eukcc:
         tg = tog("guppy", pplaceOutReduced, togTree, touch=self.cfg["touch"])
         if tg.doIneedTorun(self.cfg["force"]):
             logging.debug("Fetching tree")
-            tg.run()
+            r = tg.run()
+            if r is False:
+                logging.debug("No placement found")
+                self.write_outfile()
 
         logging.debug("Getting best placements")
         # save path to togtree for plotting later
@@ -476,6 +555,23 @@ class eukcc:
 
         logging.info("MAG succesfully placed in tree")
 
+    def node_placement(self, node):
+        sets = self.getSets()
+        nodes = node.split(",")
+        placements = []
+        for s in sets:
+            if s["node"] in nodes:
+                placements.append(s.copy())
+        for pl in placements:
+            pl["cover"] = "NA"
+            pl["nPlacements"] = "NA"
+        self.placements = {self.cfg["placementMethod"]: placements}
+        if len(placements) == 0:
+            logging.error(
+                "No provided node was found valid, please check your input. Nodes can be provided as comma sperated list e.g node1,node201"
+            )
+            exit(1)
+
     def getSets(self):
         setinfo = os.path.join(self.cfg["db"], "sets", "setinfo.csv")
         # load sets and reduce to sets matching parameters
